/* ================================================================

 * Cewolf : Chart enabling Web Objects Framework

 * ================================================================

 *

 * Project Info:  http://cewolf.sourceforge.net

 * Project Lead:  Guido Laures (guido@laures.de);

 *

 * (C) Copyright 2002, by Guido Laures

 *

 * This library is free software; you can redistribute it and/or modify it under the terms

 * of the GNU Lesser General Public License as published by the Free Software Foundation;

 * either version 2.1 of the License, or (at your option) any later version.

 *

 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;

 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 * See the GNU Lesser General Public License for more details.

 *

 * You should have received a copy of the GNU Lesser General Public License along with this

 * library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,

 * Boston, MA 02111-1307, USA.

 */



package de.laures.cewolf.storage;



import java.io.File;

import java.io.FileInputStream;

import java.io.FileOutputStream;

import java.io.IOException;

import java.io.ObjectInputStream;

import java.io.ObjectOutputStream;

import java.io.Serializable;

import java.util.ArrayList;

import java.util.List;
import java.util.HashMap;

import javax.servlet.ServletContext;

import javax.servlet.http.HttpServletRequest;

import javax.servlet.jsp.PageContext;



import de.laures.cewolf.CewolfException;

import de.laures.cewolf.ChartImage;

import de.laures.cewolf.Configuration;

import de.laures.cewolf.Storage;

import de.laures.cewolf.taglib.util.KeyGenerator;



/**

 * Storage for storing images as files in the web application directory as files _chart-XXXXX.

 * Note that by default the files won't ever be removed. To remove saved images on VM exit set

 * the <code>FileStorage.deleteOnExit</code> configuration parameter to "true". For example:

 * 

 * <pre>

 *		<init-param>

 *			<param-name>storage</param-name>

 *			<param-value>de.laures.cewolf.storage.FileStorage</param-value>

 *		</init-param>

 *		<init-param>

 *				<param-name>FileStorage.deleteOnExit</param-name>

 *				<param-value>true</param-value>

 *		</init-param> 

 *	</pre> 

 * 

 * @author guido

 */

public class FileStorage implements Storage 
{
	
	String basePath = null;
	List stored = new ArrayList();
	String fileKey = "chart";
	static HashMap fileWatcherMap=new HashMap();
	
	private boolean deleteOnExit = false;



	/**

	 * @see de.laures.cewolf.Storage#storeChartImage(ChartImage, PageContext)

	 */

	public String storeChartImage(ChartImage cid, PageContext pageContext) {

		if(contains(cid, pageContext)){

			return getKey(cid);

		}

		String id = getKey(cid);

		ObjectOutputStream oos = null;

		try {

			String fileName = getFileName(id);

			pageContext.getServletContext().log("Storing image to file " + fileName);

			File f = new File(fileName);

			if (deleteOnExit) {

				f.deleteOnExit();			

			}

			oos = new ObjectOutputStream(new FileOutputStream(f));

			oos.writeObject(new SerializableChartImage(cid));

			oos.close();

		} catch(IOException ioex){

			ioex.printStackTrace();

		} catch(CewolfException cwex){

			cwex.printStackTrace();

		} finally {

			if(oos != null){

				try {

					oos.close();

				} catch(IOException ioex){

					ioex.printStackTrace();

				}

			}

		}

		return id;		

	}



	/**

	 * @see de.laures.cewolf.Storage#getChartImage(String, HttpServletRequest)

	 */

	public ChartImage getChartImage(String id, HttpServletRequest request) {

		ChartImage res = null;

		ObjectInputStream ois = null;

		try {

			ois = new ObjectInputStream(new FileInputStream(getFileName(id)));

			res = (ChartImage)ois.readObject();

			ois.close();

		} catch(Exception ex){

			ex.printStackTrace();

		} finally {

			if(ois != null){

				try {

					ois.close();

				} catch(IOException ioex){

					ioex.printStackTrace();

				}

			}

		}

		return res;

	}



	/**

	 * @see de.laures.cewolf.Storage#contains(ChartImage, PageContext)

	 */

	public boolean contains(ChartImage chartImage, PageContext pageContext) {

		return new File(getFileName(chartImage)).exists();

	}



	/**

	 * @see de.laures.cewolf.Storage#getKey(ChartImage)

	 */

	public String getKey(ChartImage chartImage) {

		return String.valueOf(KeyGenerator.generateKey((Serializable)chartImage));

	}



	/**

	 * @see de.laures.cewolf.Storage#init(ServletContext)

	 */

	/*
	public void init(ServletContext servletContext) throws CewolfException 
	{
		basePath = servletContext.getRealPath("/");

		Configuration config = Configuration.getInstance(servletContext);

		deleteOnExit = "true".equalsIgnoreCase("" + config.getParameters().get("FileStorage.deleteOnExit"));

		servletContext.log("FileStorage initialized, deleteOnExit=" + deleteOnExit);

	}

	*/

	public void init(ServletContext servletContext) throws CewolfException
	{
		basePath=servletContext.getRealPath("/");
		String folder=servletContext.getInitParameter("folder");
		
		if(folder==null || (folder.equalsIgnoreCase("")))
		{
			folder="cewolf_charts"+File.separator;
		}

		folder=servletContext.getRealPath(folder);
		//folder=basePath+File.separator+folder+File.separator;

		File f=new File(folder);
		if(f.isDirectory())
		{
			basePath=folder;
			startWatcher();
			return;
		}
		
		if(f.mkdir())
		{
			basePath=folder;
		}
		else
		{
			System.out.println("FileStorage:problem while creating temporary folder..."+folder);
		}
			startWatcher();
	
	}

	private  void startWatcher()
	{
		//this fix is to avoid multiple threads... because servlets are reloaded periodically...
		//this static hashmap is to support multiple instances of file storage in the same vm...
		if(!fileWatcherMap.containsKey(basePath))
		{
			//System.out.println("FileStorage:Starting file cleaner...");
			new FileWatcher().start();
		}
	}

	private String getFileName(ChartImage chartImage){

		return getFileName(getKey(chartImage));

	}



	private String getFileName(String id){

		String path = basePath + File.separator +fileKey + id;
		//return basePath + "_chart" + id;
		return path;
	}



	/**

	 * @see de.laures.cewolf.Storage#removeChartImage(java.lang.String, javax.servlet.jsp.PageContext)

	 */

	public String removeChartImage(String imgKey, HttpServletRequest pageContext) throws CewolfException {

		File file = new File(getFileName(imgKey));

		if (file.exists())

		{

			if (!file.delete())

			{

				throw new CewolfException("Could not delete file " + file.getAbsolutePath());

			}

		}

		return imgKey;

	}


	class FileWatcher extends Thread
	{
		boolean isfirsttime=true;
		long sleepTime=10*60*1000L;
		public FileWatcher()
                {
                    super("Cewolf-filestoreCleaner");
                    setDaemon(true);
                }
		public void run()
		{
			fileWatcherMap.put(basePath,this);
			while(true)
			{
				try
				{
					if(!isfirsttime)
					{
						Thread.currentThread().sleep(sleepTime); //sleep for 10 min 
					}
					//get dir listing and delete the files older than last 10 minutes...
					long baseTime=(System.currentTimeMillis()-sleepTime);
					File baseDirectory=new File(basePath);
					File[] files=baseDirectory.listFiles();
					for(int i=0; i<files.length;i++)
					{
						File file=files[i];
						String fName=file.getName();
						//System.out.println("FileStorage:processing file<->"+file.getName());
						if(file.lastModified()< baseTime && fName.startsWith(fileKey))
						{
							try
							{
								boolean flag=file.delete();
								//System.out.println("FileStorage:deleting file<->"+file.getName()+"<->"+flag);
							}catch(Exception e)
							{
								System.out.println("FileStorage:error while deleting file<->"+file.getName());
								e.printStackTrace();
							}
						}
						else
						{
							//System.out.println("FileStorage:file we did not generate...<->"+file);
						}
					}
				}catch(Exception e)
				{
					e.printStackTrace();
				}
				finally
				{
					isfirsttime=false;
				}
			}
		}
	}


}

